@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: braces.c,v 1.19 1998/03/10 05:25:55 will Exp $";
+static char rcsid[] = "$Id: braces.c,v 1.26 1999/02/15 07:59:32 will Exp $";
 /*
  * brace.c: csh brace expansion -- includes pieces of
  * [t]csh's sh.misc.c and sh.glob.c.
@@ -9,8 +9,11 @@ static char rcsid[] = "$Id: braces.c,v 1.19 1998/03/10 05:25:55 will Exp $";
  * the resulting string.  This lets us take "a,b,c" and act as if
  * it was "{a,b,c}".
  *	- Will Deich, Jan 96.
+ * Modified to supply blkprint() routine.
+ *	- Will Deich, May 98.
  */
 
+#include <stdio.h>
 #include "version.h"
 
 /*-
@@ -82,6 +85,8 @@ static char *strsavew();
 void blkfree();
 static int blklen();
 
+void blkprint();
+
 int
 globbraces(v, wrap, blp)
 char *v;
@@ -272,7 +277,7 @@ blkfree(av0)
     if (!av0)
 	return;
     for (; *av; av++)
-	xfree((ptr_t) * av);
+	xfree((ptr_t) *av);
     xfree((ptr_t) av0);
 }
 
@@ -286,6 +291,10 @@ char **blk;
     char **duplicate;
     duplicate = (char **) xmalloc((size_t) sizeof(char *) * (len + 1));
 
+    /* Initialize duplicate[], so allocation failures are handled properly */
+    for (i=0; i<len; i++) {
+	duplicate[i] = NULL;
+    }
     for (i=0; i<len; i++) {
 	duplicate[i] = (char *) xmalloc((size_t) strlen(blk[i]) + 1);
 	if (!duplicate[i]) {
@@ -309,6 +318,20 @@ blklen(av)
     return i;
 }
 
+void
+blkprint(av)
+char **av;
+{
+    int i;
+    fprintf(stderr, "blkprint(%#lx):\n", av);
+    if (!av)
+	return;
+    for (i = 0;  *av; av++, i++) {
+	fprintf(stderr, "%d: %#lx\t<", i, (unsigned long) av[i]);
+	fprintf(stderr, "%s>\n", av[i]);
+    }
+}
+
 #ifdef TEST
 #include <stdio.h>
 
