@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: checks.c,v 1.52 1998/04/07 08:55:16 will Exp $";
+static char rcsid[] = "$Id: checks.c,v 1.53 1998/04/14 17:06:38 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -795,7 +795,7 @@ int isglobal;	/* !0 means its from a global def */
      */
 
     int invert, match;
-    int check_ugh __P((char *, char *));
+    int check_ugh P__((char *, char *));
 
     for (match=0, sl=sl->next; sl && !match; sl=sl->next) {
 	invert = *sl->pat == '!';
