@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: error.c,v 1.61 1998/03/02 06:53:59 will Exp $";
+static char rcsid[] = "$Id: error.c,v 1.62 1998/04/14 17:06:43 will Exp $";
 
 /*
  *	Copyright (c) 1993 by California Institute of Technology.
@@ -161,8 +161,8 @@ int error_tag = 0;
 #endif
 
 #if 1
-void ropenlog __P(( char *ident, int logopt, int facility, char *host ));
-void rsyslog __P(( unsigned int level, char *fmt, ... ));
+void ropenlog P__(( char *ident, int logopt, int facility, char *host ));
+void rsyslog P__(( unsigned int level, char *fmt, ... ));
 
 #define OpenLog(prog, opt, fac) ropenlog((prog), (opt), (fac), error_rlog_host)
 #define SysLog(pri, buf) rsyslog((pri), (buf))
