@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: gsgroups.c,v 1.6 1998/03/02 06:53:59 will Exp $";
+static char rcsid[] = "$Id: gsgroups.c,v 1.7 1998/04/14 17:06:45 will Exp $";
 /* This silly little file is here because gcc messes up badly with
  * prototyping the getgroups() / setgroups() functions on SunOS.
  * The problem is that on SunOS 4.x, there are two versions of each of
@@ -24,7 +24,7 @@ int Setgroups(n, g)
 int n;
 void *g;
 {
-    int setgroups __P((int, void *));
+    int setgroups P__((int, void *));
     return setgroups(n, g);
 }
  
@@ -32,7 +32,7 @@ int Getgroups(n, g)
 int n;
 void *g;
 {
-    int getgroups __P((int, void *));
+    int getgroups P__((int, void *));
     return getgroups(n, g);
 }
 #endif
