@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: options.c,v 1.60 1998/04/05 20:21:35 will Exp $";
+static char rcsid[] = "$Id: options.c,v 1.72 1999/02/15 10:18:19 will Exp $";
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* Process a global xxx=yyy option */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -6,12 +6,15 @@ static char rcsid[] = "$Id: options.c,v 1.60 1998/04/05 20:21:35 will Exp $";
 #include "super.h"
 #include "options.h"
 
-/* KEEP SORTED IN INCREASING ORDER! */
+/* KEEP SORTED IN INCREASING ORDER! (If you don't, we do sort the list
+ * later, but you may as well do your bit to decrease entropy :-)
+ */
 
 Option opts[] = {
 	{ "DIE",		LOCAL,		option_die },	/* OBSOLETE */
 	{ "addgroups",		LIST|GLOBAL|LOCAL,	option_groups },
 	{ "arg",		IS_ROOT|LOCAL,	option_arg },
+	{ "argv0",		LOCAL,		option_argv0 },
 	{ "cd",			GLOBAL|LOCAL,	option_cd },
 	{ "die",		LOCAL,		option_die },
 	{ "env",		LIST|GLOBAL|LOCAL,	option_env },
@@ -37,7 +40,7 @@ Option opts[] = {
 	{ "relative_path",	GLOBAL,		option_relative_path },
 	{ "renewtime",		GLOBAL,		option_renewtime },
 	{ "rlog_host",		GLOBAL,		option_rlog_host },
-	{ "setenv",		LOCAL,		option_setenv },
+	{ "setenv",		GLOBAL|LOCAL,	option_setenv },
 	{ "syslog",		GLOBAL,		option_syslog },
 	{ "timeout",		GLOBAL|LOCAL,	option_timeout },
 	{ "timestampbyhost",	GLOBAL,		option_timestampbyhost },
@@ -167,7 +170,6 @@ options cannot be negated: <%s>\n", word);
 		DoDebug();
 		return Error(0, 0, "%tMissing `%c'.\n", i);
 	    }
-				 
 	    optionlist = globlist;
 	}
 	return opt->process(word, value, isglobal);
@@ -244,6 +246,9 @@ option_local_clear_settings()
     if (localinfo.chdir_path)
 	free(localinfo.chdir_path);
     localinfo.chdir_path = NULL;
+    if (localinfo.argv0)
+	free(localinfo.argv0);
+    localinfo.argv0 = NULL;
     *localinfo.user = *localinfo.group = *localinfo.u_g = '\0';
     if (localinfo.env)
 	blkfree(localinfo.env);
@@ -361,6 +366,22 @@ int isglobal;
     return 0; 
 }
 
+static int
+option_argv0(word, s, isglobal)
+char *word;
+char *s;
+int isglobal;
+{
+    if (debug)
+	fprintf(stderr, "\toption:argv0=%s (%s)\n",
+				s, isglobal ? "global" : "local");
+
+    localinfo.argv0 = strdup(s);
+    if (!localinfo.argv0)
+	return Error(0, 0, "Failed to malloc space for argv0=<%s>\n", s);
+    return 0;
+}
+
 static int
 option_cd(word, s, isglobal)
 char *word;
@@ -979,20 +1000,39 @@ int isglobal;
 	OPTION_SEP, OPTION_SEP, s);
 
     /* Append this variable: first skip past already-set variables... */
-    for (i=0, p=localinfo.setenv; p[i] && i < MAXSETENV; i++)
-	;
-    /* ...then add this definition */
-    if (i == MAXSETENV) {
-	return Error(0, 0,
-		"%t\n\ttoo many setenv%cvar=xxx entries; max is %d.\n",
-		OPTION_SEP, MAXSETENV);
-    } else {
-	char *t = strdup(s);
-	if (!t)
+    if (isglobal) {
+	for (i=0, p=globalinfo.setenv; p[i] && i < MAXSETENV; i++)
+	    ;
+	/* ...then add this definition */
+	if (i == MAXSETENV) {
 	    return Error(0, 0,
-	    "Failed to malloc space for setenv using <%s>\n", s);
-	localinfo.setenv[i++] = t;
-	localinfo.setenv[i] = NULL;
+		"%t\n\ttoo many global setenv%cvar=xxx entries; max is %d.\n",
+		OPTION_SEP, MAXSETENV);
+	} else {
+	    char *t = strdup(s);
+	    if (!t)
+		return Error(0, 0,
+		"Failed to malloc space for setenv using <%s>\n", s);
+	    globalinfo.setenv[i++] = t;
+	    globalinfo.setenv[i] = NULL;
+	}
+    } else {
+	/* islocal */
+	for (i=0, p=localinfo.setenv; p[i] && i < MAXSETENV; i++)
+	    ;
+	/* ...then add this definition */
+	if (i == MAXSETENV) {
+	    return Error(0, 0,
+		"%t\n\ttoo many local setenv%cvar=xxx entries; max is %d.\n",
+		OPTION_SEP, MAXSETENV);
+	} else {
+	    char *t = strdup(s);
+	    if (!t)
+		return Error(0, 0,
+		"Failed to malloc space for setenv using <%s>\n", s);
+	    localinfo.setenv[i++] = t;
+	    localinfo.setenv[i] = NULL;
+	}
     }
     return 0;
 }
