@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: super.c,v 1.450 1998/04/07 07:30:23 will Exp $";
+static char rcsid[] = "$Id: super.c,v 1.462 1999/02/15 10:18:37 will Exp $";
 /* The code should compile with either ANSI C or K&R compilers. */
 
 /*
@@ -70,7 +70,8 @@ static char rcsid[] = "$Id: super.c,v 1.450 1998/04/07 07:30:23 will Exp $";
 
  */
 
-char superfile_writable[] = SUPERFILE;	/* The super.tab file -- put it
+char superfile_writable[MAXPATHLEN+100]
+			= SUPERFILE;	/* The super.tab file -- put it
 					 * into a char array so we can
 					 * overwrite it later, if nec.
 					 */
@@ -125,6 +126,11 @@ GlobalInfo globalinfo = {
     	{0},		/* groups[]: gid's of supplementary groups */
     	GROUPS_NOTSET,	/* ngroups: number of supplementary groups */
     	0,		/* groups_added */
+	{NULL},		/* first element of the setenv array.  We keep this
+			 * at end of global struct, so that we don't have
+			 * to initialize every element in order to reach other
+			 * struct elements.
+			 */
 	};
 
 /* The list of currently open files */
@@ -147,10 +153,10 @@ LocalInfo localinfo = {
 					 */
 		};
 
-extern char *s_re_comp __P(( char *));	/* regular-expression compiler */
-int shell_compare __P(( char * ));	/* s_re_comp()-style i/f to wildmat() */
-extern int s_re_exec __P(( char * ));	/* regular-expression comparison */
-char *shell_compile __P(( char * ));	/* s_re_exec()-style i/f to wildmat() */
+extern char *s_re_comp P__(( char *));	/* regular-expression compiler */
+int shell_compare P__(( char * ));	/* s_re_comp()-style i/f to wildmat() */
+extern int s_re_exec P__(( char * ));	/* regular-expression comparison */
+char *shell_compile P__(( char * ));	/* s_re_exec()-style i/f to wildmat() */
 
 
 char *prog;				/* this program */
@@ -159,8 +165,8 @@ int it_came_from_cmdline=0;		/* Set by -F/-T/-U/-G/-M flags */
 int check_syntax=0;			/* Set by the -c options flag */
 
 /* Routines used to compile/match user/group/host patterns */
-char *(*pat_compile) __P((char *)) = s_re_comp;
-int (*pat_compare) __P((char *)) = s_re_exec;
+char *(*pat_compile) P__((char *)) = s_re_comp;
+int (*pat_compare) P__((char *)) = s_re_exec;
 int need_re_anchor = 1;
 
 
@@ -168,6 +174,15 @@ int need_re_anchor = 1;
 unsigned char my_qm[256];		/* our quotemarks */
 unsigned char my_cc[256];		/* our comment characters */
 
+/* Core dump restrictions were shamelessly borrowed from Wietse
+ * Venema's logdaemon code, to disable core dumps with cleartext
+ * or shadow passwords.
+ */
+#ifdef RLIMIT_CORE
+    struct rlimit old_core_limit;
+    struct rlimit new_core_limit;
+#endif
+
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
@@ -182,6 +197,13 @@ char **argv;
     extern char *error_prog;
     int iarg, givehelp, giveversion, verbosity, printvars;
 
+#ifdef RLIMIT_CORE
+    getrlimit(RLIMIT_CORE, &old_core_limit);
+    new_core_limit.rlim_cur = 0;
+    new_core_limit.rlim_max = old_core_limit.rlim_max;
+    setrlimit(RLIMIT_CORE, &new_core_limit);
+#endif
+
     s = strrchr(argv[0], '/');
     prog = (s && *(s+1)) ? s+1 : argv[0];
     error_prog = ONETRUENAME;	/* same as prog, but used by Error() */
@@ -243,9 +265,12 @@ char **argv;
 	Error(0,0, "Checking syntax of superfile `%s'\n", superfile);
 
     /* Check if we need to switch to processing a user's super file */
-    if (cmd && strchr(cmd, ':'))
+    if (cmd && strchr(cmd, ':')) {
+	char *tcmd = strchr(cmd, ':') + 1;
 	if (user_supertab(cmd) == -1)
 	    return 1;
+	cmd = tcmd;
+    }
 
     /* Print version if explicitly requested or if giving help */
     if ((givehelp && verbosity != HELP_FACTS) || giveversion)
@@ -373,6 +398,11 @@ due to previously-reported problem)\n");
     if (localinfo.print)
 	puts(localinfo.print);
 
+#ifdef RLIMIT_CORE
+    /* Re-enable core dumps. */
+    setrlimit(RLIMIT_CORE, &old_core_limit);
+#endif
+
     if (execve(path, arglist, envp) == -1) {
 #ifdef INTERPRETER_HACK
 	if (errno == ENOEXEC) {
@@ -382,7 +412,7 @@ due to previously-reported problem)\n");
 
 	    if ((fp = fopen(path, "r")) &&		/* open the file */
 		fgets(line1, sizeof(line1), fp) &&	/* read first line */
-		(strlen(line1) < sizeof(line1)-1) &&	/* not too long? */
+		strchr(line1, '\n') &&			/* not too long? */
 		(strncmp(line1, "#!", 2) == 0) &&	/* begins "#!"? */
 		(interp = strtok(line1+2, " \t\n"))) {	/* has interpreter? */
 
@@ -549,7 +579,7 @@ int *givehelp, *giveversion, *verbosity, *printvars;
 	    /* Check for optional superfile */
 	    if (takes_optional_superfile) {
 		if (*(s+1)) {
-		    /* User gave -Xfile */
+		    /* User gave -Xfile (where X is -F/-T/-U/-G/-M) */
 		    superfile = s+1;
 		    s += strlen(s) - 1;
 		} else if ((iarg+1 < argc) && argv[iarg+1][0] != '-') {
@@ -692,9 +722,11 @@ int n_builtin;
 
     (void) fprintf(stderr,
 			"\tEnvironment variables defined with setenv=var=:\n");
-    if (localinfo.setenv[0] == NULL) {
+    if (localinfo.setenv[0] == NULL && globalinfo.setenv[0] == NULL) {
 	(void) printf("\t\t(none)\n");
     } else {
+	for (p=globalinfo.setenv; *p; p++)
+	    (void) printf("\t\t%s\n", *p);
 	for (p=localinfo.setenv; *p; p++)
 	    (void) printf("\t\t%s\n", *p);
     }
@@ -770,6 +802,11 @@ char *command;		/* user:cmd */
  
     if (pass->pw_dir[0] == '\0')
 	return Error(0, 0, "No home directory for user `%s'?!\n", user);
+    if ((strlen(pass->pw_dir) +
+		strlen(PERUSER_SUPERFILE) + 1) > sizeof(superfile_writable))
+	return Error(0, 0,
+	    "User %s's <home directory>+<%s> pathlen exceeds %d?!\n",
+	    user, PERUSER_SUPERFILE, sizeof(superfile_writable));
     (void) strcpy(superfile, pass->pw_dir);
     s = superfile + strlen(superfile) - 1;
     if (*s != '/')
@@ -791,7 +828,6 @@ char *command;		/* user:cmd */
 	return Error(1, 0,
 	    "Couldn't set uid to user %s (from command <%s:%s>): ",
 	    user, user, cmd);
-
     return 0;
 }
 
@@ -892,7 +928,7 @@ int *n_builtin;		/* returned w/ number of args in path_plus */
 	s = StrEltGetPtr(&localinfo.argpats, iarg);
 	if (s && match_pattern(0, 0, argv[iarg], s) != 1)
 	    (void) Error(0, 2,
-		"Your argument #%d <%s> must match pattern <%s>\n",
+		"Your argument #%d <%-.500s> must match pattern <%s>\n",
 		iarg, argv[iarg], s);
     }
 
@@ -911,13 +947,23 @@ int *n_builtin;		/* returned w/ number of args in path_plus */
      */
     arglist += 2;
 
-    /* First copy the command to the new arglist */
-    arglist[0] = *argv++;
-
-    /* Then copy the extra args from super.tab to the arglist,
+    /* Copy the extra args from super.tab to the arglist,
      * re-allocing the arglist as the number of args grows.
+     * First set up arglist[0]: depending on the value of localinfo.argv0,
+     * either copy argv[0] to the arglist, or use the super.tab-supplied value.
      */
-    s=strqtokS(path_plus, SEP, QM, "", 1); /* Don't put FullPath into arglist */
+    s=strqtokS(path_plus, SEP, QM, "", 1); /* FullPath */
+    if (localinfo.argv0 && (strcmp(localinfo.argv0, "<path>") == 0)) {
+	/* Use the actual path */
+	arglist[0] = s;
+    } else if (localinfo.argv0) {
+	/* Use the path named in the argv0 option in the super.tab file */
+	arglist[0] = localinfo.argv0;
+    } else {
+	/* Use the name by which this command was invoked. */
+	arglist[0] = *argv++;
+    }
+
     for(nxtra=0, ap = &arglist[1], s=strqtokS(NULL, SEP, NULL, NULL, 1);  s;
 				s = strqtokS(NULL, SEP, NULL, NULL, 1)) {
 	nxtra++;
@@ -967,7 +1013,7 @@ char *cmd;		/* name of command being started */
      *  NULL on error;
      *	otherwise, a pointer to the modified environment list.
      */
-    int i, fd, maxfd;
+    int i, j, n, fd, maxfd;
     char **p, *s;
 
     int fd_log;
@@ -1062,22 +1108,60 @@ char *cmd;		/* name of command being started */
     env[i++] = OrigHome;
 
     /* Now add the extra environment variables requested in the
-     * super.tab file.
+     * super.tab file.  Make a linear search each time, so that we
+     * don't define the same name twice.  Yes, this will be extraordinarily
+     * inefficient if there are many variables, but in real life, there
+     * aren't too many to do this...
      */
     for (p=localinfo.env; p && *p && i < NELEM(env)-1; p++) {
-	env[i] = Getenv(*p);
-	if (env[i]) {
+	n = strlen(*p);
+	for (j=0; j < i; j++) {
+	    if (strncmp(env[j], *p, n) == 0 && env[j][n] == '=') {
+		/* name is already in use, at index j. */
+		break;
+	    }
+	}
+	s = Getenv(*p);
+	if (s) {
+	    env[j] = s;
 	    if (strlen(env[i])+1 > localinfo.maxenvlen) {
 		Error(0, 0, "%t\n\tDefinition for envvar %s exceeds \
 maxenvlen=%d.\n", *p, localinfo.maxenvlen);
 		return NULL;
 	    }
+	    if (j == i) {
+		i++;
+	    }
+	}
+    }
+
+    for (p = globalinfo.setenv; *p && i < NELEM(env)-1 ; ) {
+	n = strchr(*p, '=') + 1 - *p;
+	for (j=0; j < i; j++) {
+	    if (strncmp(env[j], *p, n) == 0) {
+		/* name is already in use, at index j. */
+		break;
+	    }
+	}
+	env[j] = *p++;
+	if (j == i) {
 	    i++;
 	}
     }
 
-    for (p = localinfo.setenv; *p && i < NELEM(env)-1 ; )
-	env[i++] = *p++;
+    for (p = localinfo.setenv; *p && i < NELEM(env)-1 ; ) {
+	n = strchr(*p, '=') + 1 - *p;
+	for (j=0; j < i; j++) {
+	    if (strncmp(env[j], *p, n) == 0) {
+		/* name is already in use, at index j. */
+		break;
+	    }
+	}
+	env[j] = *p++;
+	if (j == i) {
+	    i++;
+	}
+    }
 
     if (i >= NELEM(env)-1) {
 	Error(0, 0, "%t\n\tAsked to save too many \
@@ -1156,7 +1240,11 @@ add_builtin_variables()
 void
 init_globalinfo()
 {
+    int i;
     init_umask(1);
+
+    for (i=0; i<=MAXSETENV; i++)
+	globalinfo.setenv[i]=NULL;
 }
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
