@@ -8,9 +8,9 @@
  *    License or the Artistic License, as specified in the README file.
  */
 
-/* Use __P() to have prototypes in STD C code, and not use
+/* Use P__() to have prototypes in STD C code, and not use
  * prototypes in K&R C: declare functions as:
- *	func_type  funcname __P((arglist));
+ *	func_type  funcname P__((arglist));
  */
 
 /* ==================================================================== */
@@ -68,7 +68,7 @@
 #endif
 
 #ifndef MAXFD
-int getdtablesize __P(( void ));
+int getdtablesize P__(( void ));
 #define MAXFD (getdtablesize()-1)
 #endif
 
@@ -113,7 +113,7 @@ extern int innetgr();
 #endif
 
 #ifndef HAVE_MEMSET
-void *memset __P(( void *s, int c, size_t n ));
+void *memset P__(( void *s, int c, size_t n ));
 #endif
 
 /* special code to indicate we haven't specified a uid yet */
@@ -182,8 +182,8 @@ void *memset __P(( void *s, int c, size_t n ));
 /*
  * Extern variable and routines used to compile/match user/group/host patterns.
  */
-extern char *(*pat_compile) __P(( char *));
-extern int (*pat_compare) __P(( char *));
+extern char *(*pat_compile) P__(( char *));
+extern int (*pat_compare) P__(( char *));
 extern int need_re_anchor;
 /* ========================================================================= */
 
@@ -351,6 +351,8 @@ struct globalInfo {
     GETGROUPS_T groups[NGROUPS_MAX];	/* groups from [add]groups=xxx,... */
     int ngroups;		/* number of supplementary groups */
     int groups_added;		/* bool: groups were addgroups=, not groups= */
+
+    char *setenv[MAXSETENV+1];	/* values of setenv=var[,...] option */
 };
 typedef struct globalInfo GlobalInfo;
 
@@ -386,6 +388,7 @@ struct localInfo {
     char *die;		/* Gets msg from die=msg ; null if none */
     char *print;	/* Gets msg from print=msg ; null if none */
     char *chdir_path;	/* Change to this dir before exec'ing; null if none */
+    char *argv0;	/* value of argv0=xxx option */
 
     char user[32];	/* value of uid=xxx option */
     char group[32];	/* value of gid=xxx option */
@@ -433,119 +436,120 @@ extern LocalInfo localinfo;		/* per-cmd info */
 extern Conditions matches;		/* To keep track of what matched */
 
 /* ========================================================================= */
-void	add_builtin_variables __P(( void ));
+void	add_builtin_variables P__(( void ));
 #ifdef HAVE_SYSINFO
-void	add_sysinfo_variables __P(( void ));
+void	add_sysinfo_variables P__(( void ));
 #endif
 #ifdef HAVE_UNAME
-void	add_uname_variables __P(( void ));
+void	add_uname_variables P__(( void ));
 #endif
-int	add_variable __P(( char *varname, char *vardefn ));
-void	anchor __P(( char *in, char *out ));
-char*	approve __P((char *usrcmd, int givehelp, int verbose));
-int	blkfree __P((char **));
-char ** blkdup __P((char **blk));
-int	build_cmd_file __P((char *, int , char *, char **));
-char**	buttonup __P((char *));
-int	check_arglistlen __P(( char **argv ));
-int	check_owner __P(( void ));
-int	check_pass __P(( char *cmd ));
-int	checkenv __P(( char *name,  char *value,  char *pat ));
-char*	clean_buf __P(( char *buf, char *outbuf ));
-void	close_writer __P(( void ));
-int	colon_define __P(( char *command ));
-int	colon_global __P(( char *command ));
-int	colon_include __P(( char *command, int allow_missing ));
-int	colon_if __P(( char *command, int *colonif_succeeded ));
-int	conditions_and_options __P((char *cond_or_opt_wd));
-char*	dayname __P(( int daynum ));
-int	daynum __P(( int unixtime ));
-void	debug_print __P((char *path, char **arglist, char **envp, int n_bltin));
-void	debug_hello __P((void));
-int	do_options __P((int argc, char **argv, int *help, int *vers,
+int	add_variable P__(( char *varname, char *vardefn ));
+void	anchor P__(( char *in, char *out ));
+char*	approve P__((char *usrcmd, int givehelp, int verbose));
+char ** blkdup P__((char **blk));
+int	blkfree P__((char **));
+int	blkprint P__((char **));
+int	build_cmd_file P__((char *, int , char *, char **));
+char**	buttonup P__((char *));
+int	check_arglistlen P__(( char **argv ));
+int	check_owner P__(( void ));
+int	check_pass P__(( char *cmd ));
+int	checkenv P__(( char *name,  char *value,  char *pat ));
+char*	clean_buf P__(( char *buf, char *outbuf ));
+void	close_writer P__(( void ));
+int	colon_define P__(( char *command ));
+int	colon_global P__(( char *command ));
+int	colon_include P__(( char *command, int allow_missing ));
+int	colon_if P__(( char *command, int *colonif_succeeded ));
+int	conditions_and_options P__((char *cond_or_opt_wd));
+char*	dayname P__(( int daynum ));
+int	daynum P__(( int unixtime ));
+void	debug_print P__((char *path, char **arglist, char **envp, int n_bltin));
+void	debug_hello P__((void));
+int	do_options P__((int argc, char **argv, int *help, int *vers,
 					int *verbose, int *printvars));
-char*	do_variables __P(( char *str));
-void	print_variable __P(( int indx, char *key, char *data ));
-char*	ends __P(( char *s1, char *s2 ));
+char*	do_variables P__(( char *str));
+void	print_variable P__(( int indx, char *key, char *data ));
+char*	ends P__(( char *s1, char *s2 ));
 #ifdef HAVE_STDARG_H
-int	Error __P(( int show_perror, int die, char * fmt, ... ));
+int	Error P__(( int show_perror, int die, char * fmt, ... ));
 #else
 int	Error();
 #endif
-FileList* file_open __P(( FileList *list, char *name, int allow_missing ));
-FileList* file_close __P(( FileList *list ));
-char*	fillbuffer __P(( FILE *fp, int *indentok, int *nl ));
-int	findgid __P(( int allowbrackets, char *grouplabel ));
-int	findgroup __P(( char *grouplabel ));
-int	fixup_fullpath __P((int , char *, char *, char *, int ));
-void	free_SimpleList __P(( SimpleList *));
-void	free_Simple2List __P(( Simple2List *));
-void	free_TimeList __P(( TimeList *));
-int	get_canonical_hostname __P((char *buf, int len));
-int	get_encrypted_pw __P(( void ));
-int	get_owner __P((char *path, uid_t *uid, gid_t *gid));
-int	get_password __P(( char *cmd, char *user,  char *salt,  char *encr ));
-char*	Getenv __P(( char *s ));
-int	Getgroups __P((int, GETGROUPS_T *));
-int	getlogdir __P((char *user, char *buf));
+FileList* file_open P__(( FileList *list, char *name, int allow_missing ));
+FileList* file_close P__(( FileList *list ));
+char*	fillbuffer P__(( FILE *fp, int *indentok, int *nl ));
+int	findgid P__(( int allowbrackets, char *grouplabel ));
+int	findgroup P__(( char *grouplabel ));
+int	fixup_fullpath P__((int , char *, char *, char *, int ));
+void	free_SimpleList P__(( SimpleList *));
+void	free_Simple2List P__(( Simple2List *));
+void	free_TimeList P__(( TimeList *));
+int	get_canonical_hostname P__((char *buf, int len));
+int	get_encrypted_pw P__(( void ));
+int	get_owner P__((char *path, uid_t *uid, gid_t *gid));
+int	get_password P__(( char *cmd, char *user,  char *salt,  char *encr ));
+char*	Getenv P__(( char *s ));
+int	Getgroups P__((int, GETGROUPS_T *));
+int	getlogdir P__((char *user, char *buf));
 struct passwd *
-	getpwentry __P((int allow_brackets, char *username));
-int	global_arg __P((char *word));
-int	globbraces __P(( char *s, int wrap_in_braces, char ***globlist ));
-int	handle_option __P((char *word, char *s, int isglobal));
-int	ingroup __P(( char * user, gid_t gid, char * gp_pat ));
-void	init_nice_incr __P((int is_global));
-void	init_strqtokS __P(( void ));
-void	init_umask __P((int is_global));
-void	init_globalinfo __P( (void) );
-void	init_localinfo __P( (void) );
-int	init_userinfo __P( (void) );
-int	InsertCondition __P(( char *, char *, int ));
-int	InsertTimeList __P(( char *, char **, TimeList *, char *, int ));
-int	InsertUserList __P((char *, char **, Simple2List *, SimpleList *, int));
-void	logmsg __P(( char * cmd, char ** args ));
-int	makedir __P(( char *directories ));
-char*	makedirname __P(( char *prefix, char *hostname, char *path ));
-int	match_pattern __P((int match, int glob, char *str, char *pattern));
-void	match_ugh_user __P((Simple2List *sl, int isglobal));
-void	matchtime __P(( OurTime *our, TimeList *tl ));
-char**	newargs __P((char *path_plus, char **argv, int *n_builtin));
-FILE*	open_writer __P(( char *user, char *filename, pid_t *pid_p ));
-void	opensuperlog __P(( void ));
-int	option_clear_settings __P((char *word, char *s, int isglobal));
-int	option_global_reset_settings __P((void));
-int	option_local_clear_settings __P((void));
-int	parseline __P((int givehelp, int checksyntax, char *buf, char *usrcmd));
-void	printhelp __P(( int verbose ));
-void	printhelp_hello __P(( int verbose, char *usrcmd ));
-int	process_colon_cmds __P(( char *command, int *colonif_succeeded ));
-int	process_logfile_opt __P(( void ));
-int	rcl_nice_incr __P((void));
-int	rcl_umask __P((void));
-void	re_anchor __P(( char *in, char *out));
-char*	s_re_comp __P(( char *));
-int	s_re_exec __P(( char *));
-void	readtime_init __P(( void ));
-int	readtimerange __P(( char *str, short *t1, short *t2, char *d ));
-int	readtime __P(( char *str, short *t1, char *d ));
-int	set_chdir __P((void));
-int	set_nice_incr __P((void));
-int	set_u_g __P(( void ));
-void	set_umask __P((void));
-int	Setgroups __P((int, GETGROUPS_T *));
-int	shell_compare __P(( char *));
-char*	shell_compile __P(( char *));
-void	store_nice_incr __P((int nice_incr, int is_global));
-void	store_umask __P((int mask, int is_global));
-char*	str_val __P(( char *left, int sep, char *str ));
-int	StrEltCpy __P(( StrArray *a, int ielt, char *str));
-char*	StrEltGetPtr __P(( StrArray *a, int ielt ));
-void	StrEltsUnused __P(( StrArray *a ));
-void	StrInit __P(( StrArray *a ));
-int	stringcopy __P(( char *to, char *from, int n));
-int	StrLastInUse __P(( StrArray *a ));
-int	StrNalloc __P(( StrArray *a, int nelt));
-int	StrNelts __P(( StrArray *a ));
-void	strtolower __P(( char *string ));
-char*	strqtokS __P(( char *, char *, char *, char *, unsigned int ));
-int	user_supertab __P(( char *command ));
+	getpwentry P__((int allow_brackets, char *username));
+int	global_arg P__((char *word));
+int	globbraces P__(( char *s, int wrap_in_braces, char ***globlist ));
+int	handle_option P__((char *word, char *s, int isglobal));
+int	ingroup P__(( char * user, gid_t gid, char * gp_pat ));
+void	init_nice_incr P__((int is_global));
+void	init_strqtokS P__(( void ));
+void	init_umask P__((int is_global));
+void	init_globalinfo P__( (void) );
+void	init_localinfo P__( (void) );
+int	init_userinfo P__( (void) );
+int	InsertCondition P__(( char *, char *, int ));
+int	InsertTimeList P__(( char *, char **, TimeList *, char *, int ));
+int	InsertUserList P__((char *, char **, Simple2List *, SimpleList *, int));
+void	logmsg P__(( char * cmd, char ** args ));
+int	makedir P__(( char *directories ));
+char*	makedirname P__(( char *prefix, char *hostname, char *path ));
+int	match_pattern P__((int match, int glob, char *str, char *pattern));
+void	match_ugh_user P__((Simple2List *sl, int isglobal));
+void	matchtime P__(( OurTime *our, TimeList *tl ));
+char**	newargs P__((char *path_plus, char **argv, int *n_builtin));
+FILE*	open_writer P__(( char *user, char *filename, pid_t *pid_p ));
+void	opensuperlog P__(( void ));
+int	option_clear_settings P__((char *word, char *s, int isglobal));
+int	option_global_reset_settings P__((void));
+int	option_local_clear_settings P__((void));
+int	parseline P__((int givehelp, int checksyntax, char *buf, char *usrcmd));
+void	printhelp P__(( int verbose ));
+void	printhelp_hello P__(( int verbose, char *usrcmd ));
+int	process_colon_cmds P__(( char *command, int *colonif_succeeded ));
+int	process_logfile_opt P__(( void ));
+int	rcl_nice_incr P__((void));
+int	rcl_umask P__((void));
+void	re_anchor P__(( char *in, char *out));
+char*	s_re_comp P__(( char *));
+int	s_re_exec P__(( char *));
+void	readtime_init P__(( void ));
+int	readtimerange P__(( char *str, short *t1, short *t2, char *d ));
+int	readtime P__(( char *str, short *t1, char *d ));
+int	set_chdir P__((void));
+int	set_nice_incr P__((void));
+int	set_u_g P__(( void ));
+void	set_umask P__((void));
+int	Setgroups P__((int, GETGROUPS_T *));
+int	shell_compare P__(( char *));
+char*	shell_compile P__(( char *));
+void	store_nice_incr P__((int nice_incr, int is_global));
+void	store_umask P__((int mask, int is_global));
+char*	str_val P__(( char *left, int sep, char *str ));
+int	StrEltCpy P__(( StrArray *a, int ielt, char *str));
+char*	StrEltGetPtr P__(( StrArray *a, int ielt ));
+void	StrEltsUnused P__(( StrArray *a ));
+void	StrInit P__(( StrArray *a ));
+int	stringcopy P__(( char *to, char *from, int n));
+int	StrLastInUse P__(( StrArray *a ));
+int	StrNalloc P__(( StrArray *a, int nelt));
+int	StrNelts P__(( StrArray *a ));
+void	strtolower P__(( char *string ));
+char*	strqtokS P__(( char *, char *, char *, char *, unsigned int ));
+int	user_supertab P__(( char *command ));
