@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: utils.c,v 1.89 1998/04/07 08:04:06 will Exp $";
+static char rcsid[] = "$Id: utils.c,v 1.90 1998/04/14 17:06:49 will Exp $";
 #include "super.h"
 
 /* Variables table flag -- !0 means has been created */
@@ -11,8 +11,8 @@ struct Ebuf {
     int nalloc;
 };
 /* checksize of Ebuf; grow an Ebuf */
-static char *checksize __P((struct Ebuf *cb, int N));
-static char *grow __P((struct Ebuf *cb, int N));
+static char *checksize P__((struct Ebuf *cb, int N));
+static char *grow P__((struct Ebuf *cb, int N));
 
 /* The input, cleaned input, and variable expansion buffers */
 static struct Ebuf ebuf = { NULL, 0, 0 };
@@ -88,7 +88,7 @@ anchor(in, out)
 char *in;
 char *out;
 {
-    void re_anchor __P((char *, char *));
+    void re_anchor P__((char *, char *));
     if (need_re_anchor)
 	re_anchor(in, out);
     else
@@ -1259,7 +1259,7 @@ int
 shell_compare(s)
 char *s;
 {
-    extern int wildmat __P((char *str, char *pat));
+    extern int wildmat P__((char *str, char *pat));
     return wildmat(s, shell_pattern);
 }
 
